from functools import reduce
from datetime import datetime


def log(message, *paramaters):
    msg = message.replace('{}', '\033[97m{}\033[0m').format(*paramaters) if paramaters else message
    print(f'[\033[2m{datetime.now():%Y-%m-%d %H:%M:%S}\033[0m] {msg}')


def compose(*functions):
    return reduce(lambda f, g: lambda x: f(g(x)), functions)



import os

from functools import partial


def only_mp3(file_name: str) -> bool:
    return file_name.endswith('.mp3')

def is_file(directory_path: str, file_name: str) -> bool:
    return os.path.isfile(os.path.join(directory_path, file_name))

def get_files_from(directory_path: str):
    return sorted(os.listdir(directory_path), reverse=True)

def get_downloaded_files(podcast_directory: str) -> [str]:
    is_directory_file = partial(is_file, podcast_directory)

    return (file
            for file in get_files_from(podcast_directory)
            if only_mp3(file) and is_directory_file(file))

def get_last_downloaded(podcast_directory: str):
    return next(get_downloaded_files(podcast_directory), None)



import os
import json
from argparse import ArgumentParser


def merge_parameters_collection(default: dict, *args) -> dict:
    result = dict(default)

    for arg in args:
        for key, value in arg.items():
            result[key] = value

    return result


def load_configuration_file(file_path: str) -> dict:
    if not os.path.isfile(file_path):
        raise Exception(f'Cannot read from configuration file "{file_path}"')

    with open(file_path) as json_file:
        return json.load(json_file)


def parse_argv(parser: ArgumentParser, args=None):
    return {key: value
            for key, value in vars(parser.parse_args(args)).items()
            if value}



import os
import urllib
import argparse
import re
import time

from functools import partial
from .utils import log, compose
from .downloaded import get_last_downloaded
from .parameters import merge_parameters_collection, load_configuration_file, parse_argv
from .rss import RSSEntity,\
        build_rss_entity,\
        RSSEntitySimpleName,\
        RSSEntityWithDate,\
        prepare_rss_data_from,\
        only_new_entities,\
        only_last_entity,\
        get_n_age_date,\
        only_entities_from_date


def download_rss_entity_to_path(path, rss_entity: RSSEntity):
    return urllib.request.urlretrieve(
        rss_entity.link,
        os.path.join(path, rss_entity.to_file_name()))


def build_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--downloads_limit",
        required=False,
        type=int,
        help='The maximum number of mp3 files which script will download')

    parser.add_argument(
        "--if_directory_empty",
        required=False,
        type=str,
        help='The general approach on empty directory'
    )

    return parser


def configuration_to_function(configuration: dict):
    configuration_value = configuration['if_directory_empty']

    if configuration_value == 'download_last':
        return only_last_entity

    from_n_day_match = re.match(r'^download_from_(\d+)_days$', configuration_value)
    if from_n_day_match:
        from_date = get_n_age_date(int(from_n_day_match[1]), time.localtime())
        return only_entities_from_date(from_date)

    raise Exception(f"The value the '{configuration_value}' is not recognizable")


if __name__ == '__main__':
    import sys

    DEFAULT_CONFIGURATION = {
        'downloads_limit': sys.maxsize,
        'if_directory_empty': 'download_last',
        'podcasts': [],
    }

    CONFIG_FILE = '~/.podcast_downloader_config.json'
    log('Loading configuration (from file: "{}")', CONFIG_FILE)

    CONFIGURATION = merge_parameters_collection(
        DEFAULT_CONFIGURATION,
        load_configuration_file(os.path.expanduser(CONFIG_FILE)),
        parse_argv(build_parser())
    )

    RSS_SOURCES = CONFIGURATION['podcasts']
    DOWNLOADS_LIMITS = CONFIGURATION['downloads_limit']

    on_directory_empty = configuration_to_function(CONFIGURATION)

    for rss_source in RSS_SOURCES:
        rss_source_name = rss_source['name']
        rss_source_path = rss_source['path']
        rss_source_link = rss_source['rss_link']
        rss_require_date = rss_source.get('require_date', False)
        rss_disable = rss_source.get('disable', False)

        if rss_disable:
            log('Skipping the "{}"', rss_source_name)
            continue

        log('Checking "{}"', rss_source_name)

        last_downloaded_file = get_last_downloaded(rss_source_path)
        log('Last downloaded file "{}"', last_downloaded_file or '<none>')

        download_limiter_function = partial(only_new_entities, last_downloaded_file) \
            if last_downloaded_file \
            else on_directory_empty

        rss_entity_builder = partial(
            build_rss_entity,
            RSSEntityWithDate if rss_require_date else RSSEntitySimpleName)

        missing_files_links = compose(
            list,
            download_limiter_function,
            partial(map, rss_entity_builder),
            prepare_rss_data_from
        )(rss_source_link)

        if missing_files_links:
            for rss_entry in reversed(missing_files_links):
                if DOWNLOADS_LIMITS == 0:
                    continue

                log('{}: Downloading file: "{}"', rss_source_name, rss_entry.link)
                download_rss_entity_to_path(rss_source_path, rss_entry)
                DOWNLOADS_LIMITS -= 1
        else:
            log('{}: Nothing new', rss_source_name)

        log('-' * 30)

    log('Finished')



import time
from dataclasses import dataclass
from functools import partial
from itertools import takewhile, islice
from typing import Dict, List, Tuple

import feedparser
from .utils import compose


@dataclass
class RSSEntity():
    published_date: time.struct_time
    link: str

@dataclass
class RSSEntitySimpleName(RSSEntity):

    def to_file_name(self) -> str:
        filename = self.link.rpartition('/')[-1].lower()
        if filename.find("?") > 0:
            filename = filename.rpartition('?')[0]
        return filename

@dataclass
class RSSEntityWithDate(RSSEntity):

    def to_file_name(self) -> str:
        podcast_name = RSSEntitySimpleName.to_file_name(self)
        return f'[{time.strftime("%Y%m%d", self.published_date)}] {podcast_name}'

def build_rss_entity(constructor, strip_rss_entry):
    return constructor(strip_rss_entry[0], strip_rss_entry[1][0].href)

def get_raw_rss_entries_from_web(rss_link: str) -> list:
    yield from feedparser.parse(rss_link).entries

def is_audio(link: Dict[str, str]) -> bool:
    return link.type == 'audio/mpeg'

def only_audio(links: List[Dict[str, str]]) -> bool:
    return filter(is_audio, links)

def strip_data(raw_rss_entry: dict) -> Tuple:
    return raw_rss_entry.published_parsed, list(only_audio(raw_rss_entry.links))

def has_entry_podcast_link(strip_rss_entry: dict) -> bool:
    return len(strip_rss_entry[1]) > 0

prepare_rss_data_from = compose( # pylint: disable=invalid-name#it's a function
    partial(filter, has_entry_podcast_link),
    partial(map, strip_data),
    get_raw_rss_entries_from_web)

def only_new_entities(from_file: str, raw_rss_entries: List[RSSEntity]) -> List[RSSEntity]:
    return takewhile(lambda rss_entity: rss_entity.to_file_name() != from_file, raw_rss_entries)

def only_last_entity(raw_rss_entries: List[RSSEntity]) -> List[RSSEntity]:
    return islice(raw_rss_entries, 1)

def is_entity_newer(from_date: time.struct_time, entity: RSSEntity):
    return entity.published_date[:3] >= from_date[:3]

def get_n_age_date(day_number: int, from_date: time.struct_time):
    return time.localtime(time.mktime(from_date) - day_number * 24 * 60 * 60)

def only_entities_from_date(from_date: time.struct_time):
    return partial(filter, partial(is_entity_newer, from_date))



import os
from setuptools import setup

def read(file_name):
    return open(os.path.join(os.path.dirname(__file__), file_name)).read()

setup(
    version=read('version'),
    name='podcast_downloader',
    author='Dawid Plocki',
    author_email='dawid.plocki@gmail.com',
    description='The script for downloading the recent mp3 from given RSS channels',
    long_description_content_type='text/markdown',
    long_description=read('README.md'),
    packages=['podcast_downloader'],
    install_requires=['feedparser'],
    url='https://github.com/dplocki/podcast-downloader',
    classifiers=[
        'Environment :: Console',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7'
    ],
    python_requires='>=3.6'
)
